#pragma once
/*
	Reference Book:
	C++ Standard Library,
	2nd Edition, by Nicolai M. Josuttis
	Page 635

	Modified by Thomas Kim
*/

#ifndef _TMP_STACK_H
#define _TMP_STACK_H

#include <deque>
#include <exception>
#include <string>

template<typename T> class Stack
{
protected:
	std::deque<T> c; // container for the elements
	std::string name;

public:
	Stack(const std::string& n="Stack") :
		name(n) { }

	virtual ~Stack()
	{
		if (!c.empty())
		{
			cerr << "\""<<name << "\" is not empty: " <<c.size()
				<<" elements"<< endl;
		}
	}

	// exception class for pop and top() with empty stack
	class ReadEmptyStack : public std::exception
	{
	protected:
		std::string msg;

	public:
		ReadEmptyStack(const std::string& m = "Read Empty Stack") : 
			msg(m) { }

		virtual const char* what() const throw()
		{
			return msg.c_str();
		}
	};

	// mumbmer of elements
	typename std::deque<T>::size_type size() const
	{
		return c.size();
	}

	// is stack empty?
	bool empty() const
	{
		return c.empty();
	}

	// push element into the stack
	void push(const T& elem)
	{
		c.push_back(elem);
	}


	// pop element out fo the stack and return its value
	T pop()
	{
		if (c.empty())
		{
			throw ReadEmptyStack("Stack Overflow - pop()");
		}

		T elem(c.back());
		c.pop_back();

		return elem;
	}

	// return value of the next element
	T& top()
	{
		if (c.empty())
		{
			throw ReadEmptyStack("Stack Overflow - top()");
		}

		return c.back();
	}
};





#endif // end of _TMP_STACK_H
